@@ -1,7 +1,7 @@
 /*
  * hbuf.c       -- History buffer implementation
  *
- * Copyright (C) 2005, 2006 Mikael Berthe <bmikael@lists.lilotux.net>
+ * Copyright (C) 2005-2008 Mikael Berthe <mikael@lilotux.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,6 +38,7 @@ typedef struct {
   // (for ex. when HBB_FLAG_PERSISTENT is set).
   struct { // hbuf_line_info
     time_t timestamp;
+    unsigned mucnicklen;
     guint  flags;
   } prefix;
 } hbuf_block;
@@ -107,29 +108,41 @@ static inline void do_wrap(GList **p_hbuf, GList *first_hbuf_elt,
   }
 }
 
-//  hbuf_add_line(p_hbuf, text, prefix_flags, width)
+//  hbuf_add_line(p_hbuf, text, prefix_flags, width, maxhbufblocks)
 // Add a line to the given buffer.  If width is not null, then lines are
 // wrapped at this length.
+// maxhbufblocks is the maximum number of hbuf blocks we can allocate.  If
+// null, there is no limit.  If non-null, it should be >= 2.
 //
 // Note 1: Splitting according to width won't work if there are tabs; they
 //         should be expanded before.
 // Note 2: width does not include the ending \0.
 void hbuf_add_line(GList **p_hbuf, const char *text, time_t timestamp,
-        guint prefix_flags, guint width)
+        guint prefix_flags, guint width, guint maxhbufblocks,
+        unsigned mucnicklen)
 {
   GList *curr_elt;
-  char *line, *end;
+  char *line;
+  guint hbb_blocksize, textlen;
   hbuf_block *hbuf_block_elt;
 
   if (!text) return;
 
+  textlen = strlen(text);
+  hbb_blocksize = MAX(textlen+1, HBB_BLOCKSIZE);
+
   hbuf_block_elt = g_new0(hbuf_block, 1);
   hbuf_block_elt->prefix.timestamp  = timestamp;
   hbuf_block_elt->prefix.flags      = prefix_flags;
+  hbuf_block_elt->prefix.mucnicklen = mucnicklen;
   if (!*p_hbuf) {
-    hbuf_block_elt->ptr  = g_new(char, HBB_BLOCKSIZE);
+    hbuf_block_elt->ptr  = g_new(char, hbb_blocksize);
+    if (!hbuf_block_elt->ptr) {
+      g_free(hbuf_block_elt);
+      return;
+    }
     hbuf_block_elt->flags  = HBB_FLAG_ALLOC | HBB_FLAG_PERSISTENT;
-    hbuf_block_elt->ptr_end_alloc = hbuf_block_elt->ptr + HBB_BLOCKSIZE;
+    hbuf_block_elt->ptr_end_alloc = hbuf_block_elt->ptr + hbb_blocksize;
   } else {
     hbuf_block *hbuf_b_prev;
     // Set p_hbuf to the end of the list, to speed up history loading
@@ -142,23 +155,68 @@ void hbuf_add_line(GList **p_hbuf, const char *text, time_t timestamp,
   }
   *p_hbuf = g_list_append(*p_hbuf, hbuf_block_elt);
 
-  if (strlen(text) >= HBB_BLOCKSIZE) {
-    // Too long
-    text = "[ERR:LINE_TOO_LONG]";
-    hbuf_block_elt->prefix.flags |= HBB_PREFIX_INFO;
-  }
-  if (hbuf_block_elt->ptr + strlen(text) >= hbuf_block_elt->ptr_end_alloc) {
+  if (hbuf_block_elt->ptr + textlen >= hbuf_block_elt->ptr_end_alloc) {
     // Too long for the current allocated bloc, we need another one
-    hbuf_block_elt->ptr  = g_new0(char, HBB_BLOCKSIZE);
+    if (!maxhbufblocks || textlen >= HBB_BLOCKSIZE) {
+      // No limit, let's allocate a new block
+      // If the message text is big, we won't bother to reuse an old block
+      // as well (it could be too small and cause a segfault).
+      hbuf_block_elt->ptr  = g_new0(char, hbb_blocksize);
+      hbuf_block_elt->ptr_end_alloc = hbuf_block_elt->ptr + hbb_blocksize;
+      // XXX We should check the return value.
+    } else {
+      GList *hbuf_head, *hbuf_elt;
+      hbuf_block *hbuf_b_elt;
+      guint n = 0;
+      hbuf_head = g_list_first(*p_hbuf);
+      // We need at least 2 allocated blocks
+      if (maxhbufblocks == 1)
+        maxhbufblocks = 2;
+      // Let's count the number of allocated areas
+      for (hbuf_elt = hbuf_head; hbuf_elt; hbuf_elt = g_list_next(hbuf_elt)) {
+        hbuf_b_elt = (hbuf_block*)(hbuf_elt->data);
+        if (hbuf_b_elt->flags & HBB_FLAG_ALLOC)
+          n++;
+      }
+      // If we can't allocate a new area, reuse the previous block(s)
+      if (n < maxhbufblocks) {
+        hbuf_block_elt->ptr  = g_new0(char, hbb_blocksize);
+        hbuf_block_elt->ptr_end_alloc = hbuf_block_elt->ptr + hbb_blocksize;
+      } else {
+        // Let's use an old block, and free the extra blocks if needed
+        char *allocated_block = NULL;
+        char *end_of_allocated_block = NULL;
+        while (n >= maxhbufblocks) {
+          int start_of_block = 1;
+          for (hbuf_elt = hbuf_head; hbuf_elt; hbuf_elt = hbuf_head) {
+            hbuf_b_elt = (hbuf_block*)(hbuf_elt->data);
+            if (hbuf_b_elt->flags & HBB_FLAG_ALLOC) {
+              if (start_of_block-- == 0)
+                break;
+              if (n == maxhbufblocks) {
+                allocated_block = hbuf_b_elt->ptr;
+                end_of_allocated_block = hbuf_b_elt->ptr_end_alloc;
+              } else {
+                g_free(hbuf_b_elt->ptr);
+              }
+            }
+            g_free(hbuf_b_elt);
+            hbuf_head = *p_hbuf = g_list_delete_link(hbuf_head, hbuf_elt);
+          }
+          n--;
+        }
+        memset(allocated_block, 0, end_of_allocated_block-allocated_block);
+        hbuf_block_elt->ptr = allocated_block;
+        hbuf_block_elt->ptr_end_alloc = end_of_allocated_block;
+      }
+    }
     hbuf_block_elt->flags  = HBB_FLAG_ALLOC | HBB_FLAG_PERSISTENT;
-    hbuf_block_elt->ptr_end_alloc = hbuf_block_elt->ptr + HBB_BLOCKSIZE;
   }
 
   line = hbuf_block_elt->ptr;
   // Ok, now we can copy the text..
   strcpy(line, text);
-  hbuf_block_elt->ptr_end = line + strlen(line) + 1;
-  end = hbuf_block_elt->ptr_end;
+  hbuf_block_elt->ptr_end = line + textlen + 1;
 
   curr_elt = g_list_last(*p_hbuf);
 
@@ -248,7 +306,7 @@ hbb_line **hbuf_get_lines(GList *hbuf, unsigned int n)
 {
   unsigned int i;
   hbuf_block *blk;
-  guchar last_persist_prefixflags = 0;
+  guint last_persist_prefixflags = 0;
   GList *last_persist;  // last persistent flags
   hbb_line **array, **array_elt;
 
@@ -274,15 +332,21 @@ hbb_line **hbuf_get_lines(GList *hbuf, unsigned int n)
       blk = (hbuf_block*)(hbuf->data);
       maxlen = blk->ptr_end - blk->ptr;
       *array_elt = (hbb_line*)g_new(hbb_line, 1);
-      (*array_elt)->timestamp = blk->prefix.timestamp;
-      (*array_elt)->flags     = blk->prefix.flags;
-      (*array_elt)->text      = g_strndup(blk->ptr, maxlen);
+      (*array_elt)->timestamp  = blk->prefix.timestamp;
+      (*array_elt)->flags      = blk->prefix.flags;
+      (*array_elt)->mucnicklen = blk->prefix.mucnicklen;
+      (*array_elt)->text       = g_strndup(blk->ptr, maxlen);
 
       if ((blk->flags & HBB_FLAG_PERSISTENT) && blk->prefix.flags) {
         last_persist_prefixflags = blk->prefix.flags;
       } else {
-        // Propagate highlighting flag
-        (*array_elt)->flags |= last_persist_prefixflags & HBB_PREFIX_HLIGHT;
+        // Propagate highlighting flags
+        (*array_elt)->flags |= last_persist_prefixflags &
+                               (HBB_PREFIX_HLIGHT_OUT | HBB_PREFIX_HLIGHT |
+                                HBB_PREFIX_INFO | HBB_PREFIX_IN);
+        //Continuation of a message - omit the prefix
+        (*array_elt)->flags |= HBB_PREFIX_CONT;
+        (*array_elt)->mucnicklen = 0;//The nick is in the first one
       }
 
       hbuf = g_list_next(hbuf);
@@ -348,4 +412,19 @@ GList *hbuf_jump_percent(GList *hbuf, int pc)
   return g_list_nth(hbuf, pc*hlen/100);
 }
 
+//  hbuf_get_blocks_number()
+// Returns the number of allocated hbuf_block's.
+guint hbuf_get_blocks_number(GList *hbuf)
+{
+  hbuf_block *hbuf_b_elt;
+  guint count = 0U;
+
+  for (hbuf = g_list_first(hbuf); hbuf; hbuf = g_list_next(hbuf)) {
+    hbuf_b_elt = (hbuf_block*)(hbuf->data);
+    if (hbuf_b_elt->flags & HBB_FLAG_ALLOC)
+      count++;
+  }
+  return count;
+}
+
 /* vim: set expandtab cindent cinoptions=>2\:2(0:  For Vim users... */
