@@ -56,6 +56,25 @@ enum findwhat {
   namesearch
 };
 
+extern char *strprintstatus[];
+
+// Note: do not change the ordering as these values are visible
+// to the user (option 'muc_print_status')!
+enum room_printstatus {
+  status_default,
+  status_none,
+  status_in_and_out,
+  status_all
+};
+
+extern char *strautowhois[];
+
+enum room_autowhois {
+  autowhois_default,
+  autowhois_off,
+  autowhois_on
+};
+
 struct role_affil {
   enum { type_role, type_affil } type;
   union {
@@ -158,7 +177,7 @@ void    roster_unsubscribed(const char *jid);
 void    buddylist_build(void);
 void    buddy_hide_group(gpointer rosterdata, int hide);
 void    buddylist_set_hide_offline_buddies(int hide);
-inline int buddylist_isset_filter(void);
+int     buddylist_isset_filter(void);
 void    buddylist_set_filter(guchar);
 guchar  buddylist_get_filter(void);
 const char *buddy_getjid(gpointer rosterdata);
@@ -170,6 +189,10 @@ void        buddy_setinsideroom(gpointer rosterdata, guint inside);
 guint       buddy_getinsideroom(gpointer rosterdata);
 void        buddy_settopic(gpointer rosterdata, const char *newtopic);
 const char *buddy_gettopic(gpointer rosterdata);
+void    buddy_setprintstatus(gpointer rosterdata, enum room_printstatus);
+enum room_printstatus buddy_getprintstatus(gpointer rosterdata);
+void    buddy_setautowhois(gpointer rosterdata, enum room_autowhois);
+enum room_autowhois buddy_getautowhois(gpointer rosterdata);
 void    buddy_settype(gpointer rosterdata, guint type);
 guint   buddy_gettype(gpointer rosterdata);
 guint   buddy_getsubscription(gpointer rosterdata);
@@ -209,6 +232,9 @@ void    foreach_group_member(gpointer groupdata,
                              void *param);
 gpointer unread_msg(gpointer rosterdata);
 
+void   unread_jid_add(const char *jid);
+GList *unread_jid_get_list(void);
+
 GSList *compl_list(guint type);
 
 #endif /* __ROSTER_H__ */
